'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = compose;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var _lodashObjectAssign = require('lodash/object/assign');

var _lodashObjectAssign2 = _interopRequireDefault(_lodashObjectAssign);

var _lodashCollectionForEach = require('lodash/collection/forEach');

var _lodashCollectionForEach2 = _interopRequireDefault(_lodashCollectionForEach);

var _lodashObjectMerge = require('lodash/object/merge');

var _lodashObjectMerge2 = _interopRequireDefault(_lodashObjectMerge);

var _ = require('.');

/**
 * Given a description object, return a stamp aka composable.
 *
 * (desc?: reactDesc || specDesc): stamp
 */
function createStamp() {
  var desc = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var specDesc = (0, _.parseDesc)(desc);

  var stamp = function stamp(options) {
    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    var instance = Object.create(specDesc.methods || {});

    (0, _lodashObjectMerge2['default'])(instance, specDesc.deepProperties);
    (0, _lodashObjectAssign2['default'])(instance, specDesc.properties, specDesc.configuration);
    Object.defineProperties(instance, specDesc.propertyDescriptors || {});

    if (specDesc.initializers) {
      specDesc.initializers.forEach(function (initializer) {
        var result = initializer.call(instance, options, { instance: instance, stamp: stamp, args: args });
        if (result) instance = result;
      });
    }

    return instance;
  };

  (0, _lodashObjectMerge2['default'])(stamp, specDesc.deepStaticProperties);
  (0, _lodashObjectAssign2['default'])(stamp, specDesc.staticProperties);
  Object.defineProperties(stamp, specDesc.staticPropertyDescriptors || {});

  return stamp;
}

/**
 * Take any number of stamps or descriptors. Return a new stamp
 * that encapsulates combined behavior. If nothing is passed in,
 * an empty stamp is returned.
 *
 * (...args?: stamp || reactDesc || specDesc): stamp
 */

function compose() {
  for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    args[_key2] = arguments[_key2];
  }

  var descs = args.map(function (arg) {
    return arg.compose || (0, _.parseDesc)(arg);
  });
  var compDesc = {};

  if (this && this.compose) {
    /**
     * Speical handling is required for statics when using
     * the ES7 stamp decorator... should we support this?
     */
    var _compose = this.compose;

    var statics = _objectWithoutProperties(this, ['compose']);

    statics && (_compose.deepStaticProperties = (0, _lodashObjectAssign2['default'])({}, _compose.deepStaticProperties, statics));
    descs.unshift(_compose);
  }

  (0, _lodashCollectionForEach2['default'])(descs, function (desc) {
    var initializers = desc.initializers;
    var methods = desc.methods;
    var properties = desc.properties;
    var staticProperties = desc.staticProperties;
    var propertyDescriptors = desc.propertyDescriptors;
    var staticPropertyDescriptors = desc.staticPropertyDescriptors;
    var deepProperties = desc.deepProperties;
    var deepStaticProperties = desc.deepStaticProperties;
    var configuration = desc.configuration;

    // Wrap React lifecycle methods
    compDesc.methods = (0, _.wrapMethods)(compDesc.methods, methods);

    // Stamp spec
    compDesc.initializers = (compDesc.initializers || []).concat(initializers).filter(function (initializer) {
      return initializer !== undefined;
    });

    (0, _lodashCollectionForEach2['default'])({ properties: properties, staticProperties: staticProperties, propertyDescriptors: propertyDescriptors, staticPropertyDescriptors: staticPropertyDescriptors }, function (val, key) {
      return val && (compDesc[key] = (0, _lodashObjectAssign2['default'])(compDesc[key] || {}, val));
    });

    (0, _lodashCollectionForEach2['default'])({ deepProperties: deepProperties, deepStaticProperties: deepStaticProperties, configuration: configuration }, function (val, key) {
      return val && (compDesc[key] = (0, _lodashObjectMerge2['default'])(compDesc[key] || {}, val));
    });
  });

  var stamp = createStamp(compDesc);
  stamp.compose = (0, _lodashObjectAssign2['default'])(compose.bind(stamp), compDesc);

  return stamp;
}

module.exports = exports['default'];